#define TGT_DDR		DDRD
#define TGT_PORT 	PORTD
#define TGT_PIN 	PIND
#define TGT_SCK		PD6 // schime bug
#define TGT_MOSI	PD3
#define TGT_MISO	PD2

#define SPI_CS_DDR	DDRC
#define SPI_CS_PORT	PORTC
#define SPI_CS		PC4

extern void init_spi();
extern void term_spi();
extern uint8_t spi_write(uint8_t c);

static inline uint8_t spi_read(void) {
	return spi_write(0x00);
}

static inline void spi_enable(void) {
	bit_clear(SPI_CS_PORT,SPI_CS);
}

static inline void spi_disable(void) {
	bit_set(SPI_CS_PORT,SPI_CS);
}

#define SERFLASH_TYPE_NONE		0x00
#define SERFLASH_TYPE_A25L080		0x12
#define	SERFLASH_TYPE_AT25DF041		0x21
#define	SERFLASH_TYPE_AT25DF081		0x22
#define	SERFLASH_TYPE_AT25DF161		0x24

#define SERFLASH_RES		0xab
#define SERFLASH_RDID		0x9f
#define SERFLASH_READ		0x03
#define SERFLASH_RDSR		0x05
#define SERFLASH_WREN		0x06
#define SERFLASH_PAGE_PGM	0x02

#define SERFLASH_SECTOR_ERASE	0x20
#define SERFLASH_BLOCK_ERASE	0xd8
#define SERFLASH_CHIP_ERASE	0xc7

#define SERFLASH_STAT_WIP	(1<<0) // Write in Progress
#define SERFLASH_STAT_WEL	(1<<1) // Write Enable Latch
#define SERFLASH_STAT_SRWD	(1<<7) // Status Register Write Disable


extern uint8_t serflash_signature[3];
extern uint8_t serflash_type;
extern uint8_t serflash_cmd;
extern uint32_t serflash_offset;
extern uint32_t serflash_size;

#ifndef SEEK_SET
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif

extern int8_t serflash_init(void);

extern void serflash_seek(int32_t offs,int8_t whence);

static inline void serflash_open(uint8_t cmd) {
	serflash_cmd = cmd;
	spi_enable();
}

static inline void serflash_reopen() {
	spi_enable();
}

static inline void serflash_close() {
	spi_disable();
}

static inline uint8_t serflash_proc(uint8_t dat) {
	serflash_offset++;
	return spi_write(dat);
}

static inline uint8_t serflash_status() {
	uint8_t ret;
	spi_enable();
	spi_write(SERFLASH_RDSR);
	ret = spi_write(0);
	spi_disable();
	return ret;
}

static inline void serflash_write_enable() {
	spi_enable();
	spi_write(SERFLASH_WREN);
	spi_disable();
}

extern void serflash_erase_4k(uint32_t offs);
